.. # ------------------( LICENSE                             )------------------
.. # Copyright (c) 2014-2025 Beartype authors.
.. # See "LICENSE" for further details.
.. #
.. # ------------------( SYNOPSIS                            )------------------
.. # URI repository (i.e., hidden reStructuredText (reST) document centralizing
.. # common URI links in reST format, automatically exposed to all other reST
.. # documents in this project via the "rst_epilog" setting in "conf.py").

.. # ------------------( IMAGES ~ badge                      )------------------
.. |bear-ified| image:: https://raw.githubusercontent.com/beartype/beartype-assets/main/badge/bear-ified.svg
   :align: top
   :target: https://beartype.readthedocs.io
   :alt: bear-ified

.. # ------------------( IMAGES ~ downstream                 )------------------
.. # Insert links to GitHub Sponsors funding at the icon level here, please!

.. # ------------------( LINKS ~ beartype : maintainer       )------------------
.. _beartype Bluesky:
   https://leycec.bsky.social
.. _@leycec:
   https://github.com/leycec
.. _leycec:
   https://github.com/leycec

.. # ------------------( LINKS ~ beartype : funding          )------------------
.. _BETSE:
   https://github.com/betsee/betse
.. _BETSEE:
   https://github.com/betsee/betsee
.. _GitHub Sponsors:
   https://github.com/sponsors/leycec
.. _beartype sponsorship:
   https://github.com/sponsors/leycec
.. _Paul Allen:
   https://en.wikipedia.org/wiki/Paul_Allen
.. _Paul Allen Discovery Center:
   http://www.alleninstitute.org/what-we-do/frontiers-group/discovery-centers/allen-discovery-center-tufts-university
.. _Paul Allen Discovery Center award:
   https://www.alleninstitute.org/what-we-do/frontiers-group/news-press/press-resources/press-releases/paul-g-allen-frontiers-group-announces-allen-discovery-center-tufts-university
.. _Paul G. Allen Frontiers Group:
   https://www.alleninstitute.org/what-we-do/frontiers-group
.. _SonarQube Advanced Security:
   https://www.sonarsource.com/products/sonarqube/advanced-security
.. _Tufts University:
   https://www.tufts.edu
.. _beadspace9:
   https://beadspace9.ca

.. # ------------------( LINKS ~ beartype : github           )------------------
.. _beartype:
   https://github.com/beartype/beartype
.. _beartype 1.0.0:
   https://github.com/beartype/beartype/issues/7
.. _beartype codebase:
   https://github.com/beartype/beartype/tree/main/beartype
.. _beartype dependents:
   https://github.com/beartype/beartype/network/dependents
.. _beartype issues:
   https://github.com/beartype/beartype/issues
.. _beartype mascot:
   https://github.com/beartype/beartype-assets/tree/main/banner
.. _beartype mascot artist:
   https://github.com/felix-hilden
.. _beartype organization:
   https://github.com/beartype
.. _beartype pulls:
   https://github.com/beartype/beartype/pulls
.. _beartype stars:
   https://github.com/beartype/beartype/stargazers
.. _beartype tests:
   https://github.com/beartype/beartype/actions?workflow=tests

.. # ------------------( LINKS ~ beartype : github : security)------------------
.. _GitHub Advisory Database:
   https://github.com/advisories
.. _beartype security:
   https://github.com/beartype/beartype/blob/main/.github/SECURITY.md

.. # ------------------( LINKS ~ beartype : github : time    )------------------
.. _beartype profiler:
   https://github.com/beartype/beartype/blob/main/bin/profile.bash
.. _beartype profiler Cisco:
   https://github.com/beartype/beartype/issues/58#issuecomment-940100279

.. # ------------------( LINKS ~ beartype : github : user    )------------------
.. _patrick-kidger:
   https://github.com/patrick-kidger
.. _harens:
   https://github.com/harens

.. # ------------------( LINKS ~ beartype : local            )------------------
.. _beartype license:
   https://github.com/beartype/beartype/blob/main/LICENSE

.. # ------------------( LINKS ~ beartype : local : module   )------------------
.. _beartype errormain:
   https://github.com/beartype/beartype/blob/main/beartype/_decor/_code/_pep/_error/errormain.py
.. _beartype pephint:
   https://github.com/beartype/beartype/blob/main/beartype/_decor/_code/_pep/_pephint.py
.. _beartype test data pep:
   https://github.com/beartype/beartype/blob/main/beartype_test/unit/data/hint/pep/proposal/
.. _beartype test data pep 484:
   https://github.com/beartype/beartype/blob/main/beartype_test/unit/data/hint/pep/proposal/data_hintpep484.py
.. _@callable_cached:
   https://github.com/beartype/beartype/blob/main/beartype/_util/cache/utilcachecall.py
.. _beartype util data pep:
   https://github.com/beartype/beartype/blob/main/beartype/_util/hint/data/pep/proposal/
.. _beartype util data pep parent:
   https://github.com/beartype/beartype/blob/main/beartype/_util/hint/data/pep/utilhintdatapep.py
.. _beartype util pep:
   https://github.com/beartype/beartype/blob/main/beartype/_util/hint/pep/proposal

.. # ------------------( LINKS ~ beartype : package          )------------------
.. _beartype Anaconda:
   https://anaconda.org/conda-forge/beartype
.. _beartype Arch:
   https://aur.archlinux.org/packages/python-beartype
.. _beartype Gentoo:
   https://packages.gentoo.org/packages/dev-python/beartype
.. _beartype Homebrew:
   https://github.com/beartype/homebrew-beartype
.. _beartype MacPorts:
   https://ports.macports.org/port/py-beartype
.. _beartype PyPI:
   https://pypi.org/project/beartype

.. # ------------------( LINKS ~ beartype : package : meta   )------------------
.. _Libraries.io:
   https://libraries.io

.. # ------------------( LINKS ~ beartype : plugin           )------------------
.. _ipython-beartype:
   https://pypi.org/project/ipython-beartype
.. _ipython-beartype GitHub:
   https://github.com/beartype/ipython-beartype
.. _pytest-beartype:
   https://pypi.org/project/pytest-beartype
.. _pytest-beartype GitHub:
   https://github.com/beartype/pytest-beartype

.. # ------------------( LINKS ~ github                      )------------------
.. _GitHub Actions:
   https://github.com/features/actions
.. _GitHub account signin:
   https://github.com/login
.. _GitHub account signup:
   https://github.com/join
.. _gitter:
   https://gitter.im

.. # ------------------( LINKS ~ idea                        )------------------
.. _Denial-of-Service:
   https://en.wikipedia.org/wiki/Denial-of-service_attack
.. _DRY:
   https://en.wikipedia.org/wiki/Don%27t_repeat_yourself
.. _IDE:
   https://en.wikipedia.org/wiki/Integrated_development_environment
.. _JIT:
   https://en.wikipedia.org/wiki/Just-in-time_compilation
.. _SQA:
   https://en.wikipedia.org/wiki/Software_quality_assurance
.. _amortized analysis:
   https://en.wikipedia.org/wiki/Amortized_analysis
.. _computer vision:
   https://en.wikipedia.org/wiki/Computer_vision
.. _continuous integration:
   https://en.wikipedia.org/wiki/Continuous_integration
.. _duck typing:
   https://en.wikipedia.org/wiki/Duck_typing
.. _dynamic typing:
   https://en.wikipedia.org/wiki/Type_system
.. _dynamically-typed:
   https://en.wikipedia.org/wiki/Type_system
.. _endorheic basins:
   https://en.wikipedia.org/wiki/Endorheic_basin
.. _full-stack:
   https://en.wikipedia.org/wiki/Solution_stack
.. _generative pre-trained transformer:
   https://en.wikipedia.org/wiki/Generative_pre-trained_transformer
.. _gratis versus libre:
   https://en.wikipedia.org/wiki/Gratis_versus_libre
.. _large language model:
   https://en.wikipedia.org/wiki/Large_language_model
.. _machine learning:
   https://en.wikipedia.org/wiki/Machine_learning
.. _memory safety:
   https://en.wikipedia.org/wiki/Memory_safety
.. _multiple dispatch:
   https://en.wikipedia.org/wiki/Multiple_dispatch
.. _near-real-time:
   https://en.wikipedia.org/wiki/Real-time_computing#Near_real-time
.. _random walk:
   https://en.wikipedia.org/wiki/Random_walk
.. _real-time:
   https://en.wikipedia.org/wiki/Real-time_computing
.. _set theory:
   https://en.wikipedia.org/wiki/Set_theory
.. _shield wall:
   https://en.wikipedia.org/wiki/Shield_wall
.. _static typing:
   https://en.wikipedia.org/wiki/Type_system
.. _statically-typed:
   https://en.wikipedia.org/wiki/Type_system
.. _topological sort:
   https://en.wikipedia.org/wiki/Topological_sorting
.. _type inference:
   https://en.wikipedia.org/wiki/Type_inference
.. _zero-cost abstraction:
   https://boats.gitlab.io/blog/post/zero-cost-abstractions

.. # ------------------( LINKS ~ kipling                     )------------------
.. _The Jungle Book:
   https://www.gutenberg.org/files/236/236-h/236-h.htm
.. _Shere Khan:
   https://en.wikipedia.org/wiki/Shere_Khan

.. # ------------------( LINKS ~ math                        )------------------
.. _Euler–Mascheroni constant:
   https://en.wikipedia.org/wiki/Euler%E2%80%93Mascheroni_constant
.. _coupon collector's problem:
   https://en.wikipedia.org/wiki/Coupon_collector%27s_problem
.. _Big O:
   https://en.wikipedia.org/wiki/Big_O_notation

.. # ------------------( LINKS ~ math : set                  )------------------
.. _conjunction:
   https://en.wikipedia.org/wiki/Logical_conjunction
.. _disjunction:
   https://en.wikipedia.org/wiki/Logical_disjunction
.. _intersection:
   https://en.wikipedia.org/wiki/Intersection_(set_theory)
.. _relative set complement:
   https://en.wikipedia.org/wiki/Complement_(set_theory)#Relative_complement

.. # ------------------( LINKS ~ math : type                 )------------------
.. _covariance:
   https://en.wikipedia.org/wiki/Covariance_and_contravariance_(computer_science)

.. # ------------------( LINKS ~ meme                        )------------------
.. _RNGesus:
   https://knowyourmeme.com/memes/rngesus
.. _Up:
   https://www.youtube.com/watch?v=F2bk_9T482g
.. _goes up to eleven:
   https://www.youtube.com/watch?v=uMSV4OteqBE
.. _greased lightning:
   https://www.youtube.com/watch?v=H-kL8A4RNQ8
.. _led zeppelin:
   https://rateyourmusic.com/release/album/led-zeppelin/led-zeppelin
.. _ludicrous speed:
   https://www.youtube.com/watch?v=6tTvklMXeFE
.. _the gripping hand:
   http://catb.org/jargon/html/O/on-the-gripping-hand.html

.. # ------------------( LINKS ~ os : linux                  )------------------
.. _Gentoo Linux:
   https://www.gentoo.org

.. # ------------------( LINKS ~ os : linux : arch           )------------------
.. _Arch Linux:
   https://archlinux.org
.. _AUR:
   https://aur.archlinux.org/packages/python-beartype

.. # ------------------( LINKS ~ os : macos                  )------------------
.. _macOS:
   https://en.wikipedia.org/wiki/MacOS
.. _HomeBrew:
   https://brew.sh
.. _MacPorts:
   https://www.macports.org

.. # ------------------( LINKS ~ other                       )------------------
.. _heliotrope:
   https://en.wikipedia.org/wiki/Heliotropium

.. # ------------------( LINKS ~ py                          )------------------
.. _Python:
   https://www.python.org
.. _Python status:
   https://devguide.python.org/versions/#versions
.. _pip:
   https://pip.pypa.io

.. # ------------------( LINKS ~ py : cli                    )------------------
.. _-O:
   https://docs.python.org/3/using/cmdline.html#cmdoption-o
.. _PYTHONOPTIMIZE:
   https://docs.python.org/3/using/cmdline.html#envvar-PYTHONOPTIMIZE

.. # ------------------( LINKS ~ py : interpreter            )------------------
.. _Brython:
   https://brython.info
.. _CPython:
   https://github.com/python/cpython
.. _Cython:
   https://cython.org
.. _Nuitka:
   https://nuitka.net
.. _PyPy:
   https://www.pypy.org
.. _Pyodide:
   https://pyodide.org

.. # ------------------( LINKS ~ py : interpreter : cpython  )------------------
.. _CPython bug tracker:
   https://github.com/python/cpython/issues

.. # ------------------( LINKS ~ py : lang                   )------------------
.. _generic alias parameters:
   https://docs.python.org/3/library/stdtypes.html#genericalias.__parameters__
.. _isinstancecheck:
   https://docs.python.org/3/reference/datamodel.html#customizing-instance-and-subclass-checks
.. _mro:
   https://docs.python.org/3/library/stdtypes.html#class.__mro__
.. _object:
   https://docs.python.org/3/reference/datamodel.html#basic-customization
.. _operator precedence:
   https://docs.python.org/3/reference/expressions.html#operator-precedence

.. # ------------------( LINKS ~ py : misc                   )------------------
.. _Guido van Rossum:
   https://en.wikipedia.org/wiki/Guido_van_Rossum
.. _RealPython:
   https://realpython.com/python-type-checking

.. # ------------------( LINKS ~ py : package                )------------------
.. _Django:
   https://www.djangoproject.com
.. _Hypothesis:
   https://hypothesis.readthedocs.io
.. _NetworkX:
   https://networkx.org
.. _PyTorch:
   https://pytorch.org
.. _SymPy:
   https://www.sympy.org
.. _TensorFlow:
   https://www.tensorflow.org
.. _gradio:
   https://www.gradio.app
.. _nptyping:
   https://github.com/ramonhagenaars/nptyping
.. _numerary:
   https://github.com/posita/numerary
.. _pyenv:
   https://operatingops.org/2020/10/24/tox-testing-multiple-python-versions-with-pyenv
.. _typing_extensions:
   https://pypi.org/project/typing-extensions
.. _xarray:
   https://docs.xarray.dev

.. # ------------------( LINKS ~ py : package : boto3        )------------------
.. _Boto3:
   https://aws.amazon.com/sdk-for-python
.. _bearboto3:
   https://github.com/beartype/bearboto3
.. _mypy-boto3:
   https://mypy-boto3.readthedocs.io

.. # ------------------( LINKS ~ py : package : click        )------------------
.. _click:
   https://click.palletsprojects.com
.. _rich_click:
   https://ewels.github.io/rich-click
.. _celery.Celery:
   https://docs.celeryq.dev/en/stable/reference/celery.html#celery.Celery
.. _celery.Celery.task:
   https://docs.celeryq.dev/en/stable/userguide/tasks.html

.. # ------------------( LINKS ~ py : package : equinox      )------------------
.. _equinox:
   https://docs.kidger.site/equinox
.. _equinox.Module:
   https://docs.kidger.site/equinox/api/module/module
.. _equinox.filter_jit:
   https://docs.kidger.site/equinox/api/transformations/#equinox.filter_jit

.. # ------------------( LINKS ~ py : package : ipython      )------------------
.. _Google Colab:
   https://colab.research.google.com
.. _IPython:
   https://ipython.org
.. _Jupyter:
   https://jupyter.org
.. _Marimo:
   https://marimo.io
.. _Zasper:
   https://zasper.io

.. # ------------------( LINKS ~ py : package : jax          )------------------
.. _jax:
   https://jax.readthedocs.io
.. _jax.jit:
   https://jax.readthedocs.io/en/latest/_autosummary/jax.jit.html#jax.jit
.. _jax.numpy:
   https://jax.readthedocs.io/en/latest/notebooks/thinking_in_jax.html
.. _jaxtyping:
   https://github.com/google/jaxtyping
.. _jaxtyping.jaxtyped:
   https://docs.kidger.site/jaxtyping/api/runtime-type-checking/#jaxtyping.jaxtyped

.. # ------------------( LINKS ~ py : package : numba        )------------------
.. _numba:
   https://numba.pydata.org
.. _numba.njit:
   https://numba.readthedocs.io/en/stable/user/jit.html

.. # ------------------( LINKS ~ py : package : numpy        )------------------
.. _NumPy:
   https://numpy.org
.. _numpy.dtype:
   https://numpy.org/doc/stable/reference/arrays.dtypes.html
.. _numpy.dtype.type:
   https://numpy.org/doc/stable/reference/arrays.dtypes.html
.. _numpy.empty_like:
   https://numpy.org/doc/stable/reference/generated/numpy.empty_like.html
.. _numpy.floating:
   https://numpy.org/doc/stable/reference/arrays.scalars.html?highlight=numpy%20generic#numpy.floating
.. _numpy.generic:
   https://numpy.org/doc/stable/reference/arrays.scalars.html?highlight=numpy%20generic#numpy.generic
.. _numpy.integer:
   https://numpy.org/doc/stable/reference/arrays.scalars.html?highlight=numpy%20generic#numpy.integer
.. _numpy.typing:
   https://numpy.org/devdocs/reference/typing.html
.. _numpy.typing.NDArray:
   https://numpy.org/devdocs/reference/typing.html#ndarray

.. # ------------------( LINKS ~ py : package : pandas       )------------------
.. # Note that "pandas" is officially lowercase.
.. _pandas:
   https://pandas.pydata.org
.. _pandas.DataFrame:
   https://pandas.pydata.org/docs/reference/api/pandas.DataFrame.html

.. # ------------------( LINKS ~ py : package : panderas     )------------------
.. # Note that "pandera" is officially lowercase.
.. _pandera:
   https://pandera.readthedocs.io
.. _pandera.check_types:
   https://pandera.readthedocs.io/en/stable/reference/generated/pandera.decorators.check_types.html
.. _pandera.typing:
   https://pandera.readthedocs.io/en/stable/reference/generated/pandera.typing.html
.. _pandera.typing.DataFrame:
   https://pandera.readthedocs.io/en/stable/dataframe_models.html

.. # ------------------( LINKS ~ py : package : pydantic     )------------------
.. _Pydantic:
   https://docs.pydantic.dev
.. _fastmcp.FastMCP:
   https://gofastmcp.com/servers
.. _fastmcp.FastMCP.tool:
   https://gofastmcp.com/servers/tools
.. _LangChain:
   https://www.langchain.com
.. _langchain_core:
   https://python.langchain.com/api_reference/core/index.html
.. _langchain_core.runnables:
   https://python.langchain.com/api_reference/core/runnables.html
.. _langchain_core.runnables.chain:
   https://python.langchain.com/api_reference/core/runnables/langchain_core.runnables.base.chain.html

.. # ------------------( LINKS ~ py : package : redis        )------------------
.. _redis:
   https://redis.readthedocs.io
.. _redis.Redis:
   https://redis.readthedocs.io/en/stable/connections.html#redis.Redis

.. # ------------------( LINKS ~ py : package : sphinx       )------------------
.. _Sphinx:
   https://www.sphinx-doc.org
.. _sphinx.ext.autodoc:
   https://www.sphinx-doc.org/en/master/usage/extensions/autodoc.html

.. # ------------------( LINKS ~ py : package : test         )------------------
.. _Codecov:
   https://about.codecov.io
.. _pytest:
   https://docs.pytest.org
.. _tox:
   https://tox.readthedocs.io

.. # ------------------( LINKS ~ py : pep                    )------------------
.. _PEPs:
   https://peps.python.org
.. _PEP 20:
   https://peps.python.org/pep-0020
.. _PEP 362:
   https://peps.python.org/pep-0362
.. _PEP 483:
   https://peps.python.org/pep-0483
.. _PEP 526:
   https://peps.python.org/pep-0526
.. _PEP 544:
   https://peps.python.org/pep-0544
.. _PEP 557:
   https://peps.python.org/pep-0557
.. _PEP 561:
   https://peps.python.org/pep-0561
.. _PEP 563:
   https://peps.python.org/pep-0563
.. _PEP 570:
   https://peps.python.org/pep-0570
.. _PEP 572:
   https://peps.python.org/pep-0572
.. _PEP 585:
   https://peps.python.org/pep-0585
.. _PEP 586:
   https://peps.python.org/pep-0586
.. _PEP 589:
   https://peps.python.org/pep-0589
.. _PEP 591:
   https://peps.python.org/pep-0591
.. _PEP 593:
   https://peps.python.org/pep-0593
.. _PEP 604:
   https://peps.python.org/pep-0604
.. _PEP 612:
   https://peps.python.org/pep-0612
.. _PEP 613:
   https://peps.python.org/pep-0613
.. _PEP 646:
   https://peps.python.org/pep-0646
.. _PEP 647:
   https://peps.python.org/pep-0647
.. _PEP 673:
   https://peps.python.org/pep-0673
.. _PEP 675:
   https://peps.python.org/pep-0675
.. _PEP 681:
   https://peps.python.org/pep-0681
.. _PEP 3102:
   https://peps.python.org/pep-3102
.. _PEP 3141:
   https://peps.python.org/pep-3141

.. # ------------------( LINKS ~ py : pep : 3119             )------------------
.. _PEP 3119:
   https://peps.python.org/pep-3119
.. _virtual base classes:
   https://peps.python.org/pep-3119/#id33

.. # ------------------( LINKS ~ py : pep : 484              )------------------
.. _PEP 484:
   https://peps.python.org/pep-0484
.. _implicit numeric tower:
   https://peps.python.org/pep-0484/#the-numeric-tower
.. _relative forward references:
   https://peps.python.org/pep-0484/#forward-references
.. _type aliases:
   https://peps.python.org/pep-0484/#type-aliases

.. # ------------------( LINKS ~ py : pep : 560              )------------------
.. _PEP 560:
   https://peps.python.org/pep-0560
.. _mro_entries:
   https://peps.python.org/pep-0560/#id20

.. # ------------------( LINKS ~ py : service                )------------------
.. _Anaconda:
   https://docs.conda.io/en/latest/miniconda.html

.. # ------------------( LINKS ~ py : service : pypi         )------------------
.. _PyPI:
   https://pypi.org
.. _PyPI cheese shop:
   https://pypi.org
.. _cheese shop sketch:
   https://en.wikipedia.org/wiki/Cheese_Shop_sketch

.. # ------------------( LINKS ~ py : stdlib                 )------------------
.. _linecache:
   https://docs.python.org/3/library/linecache.html
.. _weakref:
   https://docs.python.org/3/library/weakref.html

.. # ------------------( LINKS ~ py : stdlib : abc           )------------------
.. _abc:
   https://docs.python.org/3/library/abc.html
.. _abc.ABCMeta:
   https://docs.python.org/3/library/abc.html#abc.ABCMeta

.. # ------------------( LINKS ~ py : stdlib : builtins      )------------------
.. _builtins:
   https://docs.python.org/3/library/stdtypes.html
.. _None:
   https://docs.python.org/3/library/constants.html#None
.. _NotImplemented:
   https://docs.python.org/3/library/constants.html#NotImplemented
.. _dict:
   https://docs.python.org/3/library/stdtypes.html#mapping-types-dict
.. _dir:
   https://docs.python.org/3/library/functions.html#dir
.. _frozenset:
   https://docs.python.org/3/library/stdtypes.html#set-types-set-frozenset
.. _isinstance:
   https://docs.python.org/3/library/functions.html#isinstance
.. _issubclass:
   https://docs.python.org/3/library/functions.html#issubclass
.. _list:
   https://docs.python.org/3/library/stdtypes.html#lists
.. _memoryview:
   https://docs.python.org/3/library/stdtypes.html#memory-views
.. _range:
   https://docs.python.org/3/library/stdtypes.html#typesseq-range
.. _set:
   https://docs.python.org/3/library/stdtypes.html#set-types-set-frozenset
.. _tuple:
   https://docs.python.org/3/library/stdtypes.html#tuples
.. _type:
   https://docs.python.org/3/library/stdtypes.html#bltin-type-objects

.. # ------------------( LINKS ~ py : stdlib : collections   }------------------
.. _collections:
   https://docs.python.org/3/library/collections.html
.. _collections.ChainMap:
   https://docs.python.org/3/library/collections.html#collections.ChainMap
.. _collections.Counter:
   https://docs.python.org/3/library/collections.html#collections.Counter
.. _collections.OrderedDict:
   https://docs.python.org/3/library/collections.html#collections.OrderedDict
.. _collections.defaultdict:
   https://docs.python.org/3/library/collections.html#collections.defaultdict
.. _collections.deque:
   https://docs.python.org/3/library/collections.html#collections.deque

.. # ------------------( LINKS ~ py : stdlib : collections.abc  }---------------
.. _collections.abc:
   https://docs.python.org/3/library/collections.abc.html
.. _collections.abc.AsyncGenerator:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.AsyncGenerator
.. _collections.abc.AsyncIterable:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.AsyncIterable
.. _collections.abc.AsyncIterator:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.AsyncIterator
.. _collections.abc.Awaitable:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Awaitable
.. _collections.abc.ByteString:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.ByteString
.. _collections.abc.Callable:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Callable
.. _collections.abc.Collection:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Collection
.. _collections.abc.Container:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Container
.. _collections.abc.Coroutine:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Coroutine
.. _collections.abc.Generator:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Generator
.. _collections.abc.ItemsView:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.ItemsView
.. _collections.abc.Iterable:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Iterable
.. _collections.abc.Iterator:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Iterator
.. _collections.abc.KeysView:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.KeysView
.. _collections.abc.Mapping:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Mapping
.. _collections.abc.MappingView:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.MappingView
.. _collections.abc.MutableMapping:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.MutableMapping
.. _collections.abc.MutableSequence:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.MutableSequence
.. _collections.abc.MutableSet:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.MutableSet
.. _collections.abc.Reversible:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Reversible
.. _collections.abc.Sequence:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Sequence
.. _collections.abc.Set:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.Set
.. _collections.abc.ValuesView:
   https://docs.python.org/3/library/collections.abc.html#collections.abc.ValuesView

.. # ------------------( LINKS ~ py : stdlib : abc           )------------------
.. _dataclasses:
   https://docs.python.org/3/library/dataclasses.html
.. _dataclasses.InitVar:
   https://docs.python.org/3/library/dataclasses.html#init-only-variables
.. _dataclasses.dataclass:
   https://docs.python.org/3/library/dataclasses.html#dataclasses.dataclass

.. # ------------------( LINKS ~ py : stdlib : contextlib    )------------------
.. _contextlib:
   https://docs.python.org/3/library/contextlib.html
.. _contextlib.AbstractAsyncContextManager:
   https://docs.python.org/3/library/contextlib.html#contextlib.AbstractAsyncContextManager
.. _contextlib.AbstractContextManager:
   https://docs.python.org/3/library/contextlib.html#contextlib.AbstractContextManager

.. # ------------------( LINKS ~ py : stdlib : enum          )------------------
.. _enum.Enum:
   https://docs.python.org/3/library/enum.html#enum.Enum

.. # ------------------( LINKS ~ py : stdlib : inspect       )------------------
.. _inspect:
   https://docs.python.org/3/library/inspect.html
.. _inspect.isasyncgenfunction:
   https://docs.python.org/3/library/inspect.html#inspect.isasyncgenfunction
.. _inspect.isgeneratorfunction:
   https://docs.python.org/3/library/inspect.html#inspect.isgeneratorfunction

.. # ------------------( LINKS ~ py : stdlib : io            )------------------
.. _io:
   https://docs.python.org/3/library/io.html

.. # ------------------( LINKS ~ py : stdlib : os            )------------------
.. _os:
   https://docs.python.org/3/library/os.html
.. _os.walk:
   https://docs.python.org/3/library/os.html#os.walk

.. # ------------------( LINKS ~ py : stdlib : random        )------------------
.. _random:
   https://docs.python.org/3/library/random.html
.. _random.getrandbits:
   https://docs.python.org/3/library/random.html#random.getrandbits
.. _random twister:
   https://stackoverflow.com/a/11704178/2809027

.. # ------------------( LINKS ~ py : stdlib : re            )------------------
.. _re:
   https://docs.python.org/3/library/re.html
.. _re.Match:
   https://docs.python.org/3/library/re.html#match-objects
.. _re.Pattern:
   https://docs.python.org/3/library/re.html#regular-expression-objects

.. # ------------------( LINKS ~ py : stdlib : typing : attr )------------------
.. _typing:
   https://docs.python.org/3/library/typing.html
.. _typing.AbstractSet:
   https://docs.python.org/3/library/typing.html#typing.AbstractSet
.. _typing.Annotated:
   https://docs.python.org/3/library/typing.html#typing.Annotated
.. _typing.Any:
   https://docs.python.org/3/library/typing.html#typing.Any
.. _typing.AnyStr:
   https://docs.python.org/3/library/typing.html#typing.AnyStr
.. _typing.AsyncContextManager:
   https://docs.python.org/3/library/typing.html#typing.AsyncContextManager
.. _typing.AsyncGenerator:
   https://docs.python.org/3/library/typing.html#typing.AsyncGenerator
.. _typing.AsyncIterable:
   https://docs.python.org/3/library/typing.html#typing.AsyncIterable
.. _typing.AsyncIterator:
   https://docs.python.org/3/library/typing.html#typing.AsyncIterator
.. _typing.Awaitable:
   https://docs.python.org/3/library/typing.html#typing.Awaitable
.. _typing.BinaryIO:
   https://docs.python.org/3/library/typing.html#typing.BinaryIO
.. _typing.ByteString:
   https://docs.python.org/3/library/typing.html#typing.ByteString
.. _typing.Callable:
   https://docs.python.org/3/library/typing.html#typing.Callable
.. _typing.ChainMap:
   https://docs.python.org/3/library/typing.html#typing.ChainMap
.. _typing.ClassVar:
   https://docs.python.org/3/library/typing.html#typing.ClassVar
.. _typing.Collection:
   https://docs.python.org/3/library/typing.html#typing.Collection
.. _typing.Concatenate:
   https://docs.python.org/3/library/typing.html#typing.Concatenate
.. _typing.Container:
   https://docs.python.org/3/library/typing.html#typing.Container
.. _typing.ContextManager:
   https://docs.python.org/3/library/typing.html#typing.ContextManager
.. _typing.Coroutine:
   https://docs.python.org/3/library/typing.html#typing.Coroutine
.. _typing.Counter:
   https://docs.python.org/3/library/typing.html#typing.Counter
.. _typing.DefaultDict:
   https://docs.python.org/3/library/typing.html#typing.DefaultDict
.. _typing.Deque:
   https://docs.python.org/3/library/typing.html#typing.Deque
.. _typing.Dict:
   https://docs.python.org/3/library/typing.html#typing.Dict
.. _typing.Final:
   https://docs.python.org/3/library/typing.html#typing.Final
.. _typing.ForwardRef:
   https://docs.python.org/3/library/typing.html#typing.ForwardRef
.. _typing.FrozenSet:
   https://docs.python.org/3/library/typing.html#typing.FrozenSet
.. _typing.Generator:
   https://docs.python.org/3/library/typing.html#typing.Generator
.. _typing.Generic:
   https://docs.python.org/3/library/typing.html#typing.Generic
.. _typing.Hashable:
   https://docs.python.org/3/library/typing.html#typing.Hashable
.. _typing.IO:
   https://docs.python.org/3/library/typing.html#typing.IO
.. _typing.ItemsView:
   https://docs.python.org/3/library/typing.html#typing.ItemsView
.. _typing.Iterable:
   https://docs.python.org/3/library/typing.html#typing.Iterable
.. _typing.Iterator:
   https://docs.python.org/3/library/typing.html#typing.Iterator
.. _typing.KeysView:
   https://docs.python.org/3/library/typing.html#typing.KeysView
.. _typing.List:
   https://docs.python.org/3/library/typing.html#typing.List
.. _typing.Literal:
   https://docs.python.org/3/library/typing.html#typing.Literal
.. _typing.Mapping:
   https://docs.python.org/3/library/typing.html#typing.Mapping
.. _typing.MappingView:
   https://docs.python.org/3/library/typing.html#typing.MappinViewg
.. _typing.Match:
   https://docs.python.org/3/library/typing.html#typing.Match
.. _typing.MutableMapping:
   https://docs.python.org/3/library/typing.html#typing.MutableMapping
.. _typing.MutableSequence:
   https://docs.python.org/3/library/typing.html#typing.MutableSequence
.. _typing.MutableSet:
   https://docs.python.org/3/library/typing.html#typing.MutableSet
.. _typing.NamedTuple:
   https://docs.python.org/3/library/typing.html#typing.NamedTuple
.. _typing.NewType:
   https://docs.python.org/3/library/typing.html#typing.NewType
.. _typing.NoReturn:
   https://docs.python.org/3/library/typing.html#typing.NoReturn
.. _typing.Optional:
   https://docs.python.org/3/library/typing.html#typing.Optional
.. _typing.OrderedDict:
   https://docs.python.org/3/library/typing.html#typing.OrderedDict
.. _typing.ParamSpec:
   https://docs.python.org/3/library/typing.html#typing.ParamSpec
.. _typing.ParamSpecArgs:
   https://docs.python.org/3/library/typing.html#typing.ParamSpecArgs
.. _typing.ParamSpecKwargs:
   https://docs.python.org/3/library/typing.html#typing.ParamSpecKwargs
.. _typing.Pattern:
   https://docs.python.org/3/library/typing.html#typing.Pattern
.. _typing.Protocol:
   https://docs.python.org/3/library/typing.html#typing.Protocol
.. _typing.Reversible:
   https://docs.python.org/3/library/typing.html#typing.Reversible
.. _typing.Self:
   https://docs.python.org/3/library/typing.html#typing.Self
.. _typing.Sequence:
   https://docs.python.org/3/library/typing.html#typing.Sequence
.. _typing.Set:
   https://docs.python.org/3/library/typing.html#typing.Set
.. _typing.Sized:
   https://docs.python.org/3/library/typing.html#typing.Sized
.. _typing.SupportsAbs:
   https://docs.python.org/3/library/typing.html#typing.SupportsAbs
.. _typing.SupportsBytes:
   https://docs.python.org/3/library/typing.html#typing.SupportsBytes
.. _typing.SupportsComplex:
   https://docs.python.org/3/library/typing.html#typing.SupportsComplex
.. _typing.SupportsFloat:
   https://docs.python.org/3/library/typing.html#typing.SupportsFloat
.. _typing.SupportsIndex:
   https://docs.python.org/3/library/typing.html#typing.SupportsIndex
.. _typing.SupportsInt:
   https://docs.python.org/3/library/typing.html#typing.SupportsInt
.. _typing.SupportsRound:
   https://docs.python.org/3/library/typing.html#typing.SupportsRound
.. _typing.Text:
   https://docs.python.org/3/library/typing.html#typing.Text
.. _typing.TextIO:
   https://docs.python.org/3/library/typing.html#typing.TextIO
.. _typing.Tuple:
   https://docs.python.org/3/library/typing.html#typing.Tuple
.. _typing.Type:
   https://docs.python.org/3/library/typing.html#typing.Type
.. _typing.TypeGuard:
   https://docs.python.org/3/library/typing.html#typing.TypeGuard
.. _typing.TypedDict:
   https://docs.python.org/3/library/typing.html#typing.TypedDict
.. _typing.TypeVar:
   https://docs.python.org/3/library/typing.html#typing.TypeVar
.. _typing.Union:
   https://docs.python.org/3/library/typing.html#typing.Union
.. _typing.ValuesView:
   https://docs.python.org/3/library/typing.html#typing.ValuesView
.. _@typing.final:
   https://docs.python.org/3/library/typing.html#typing.final
.. _@typing.no_type_check:
   https://docs.python.org/3/library/typing.html#typing.no_type_check
.. _typing.TYPE_CHECKING:
   https://docs.python.org/3/library/typing.html#typing.TYPE_CHECKING

.. # ------------------( LINKS ~ py : type : runtime         )------------------
.. _enforce:
   https://github.com/RussBaz/enforce
.. _enforce_typing:
   https://github.com/matchawine/python-enforce-typing
.. _pytypes:
   https://github.com/Stewori/pytypes
.. _typeen:
   https://github.com/k2bd/typen
.. _typical:
   https://github.com/seandstewart/typical

.. # ------------------( LINKS ~ py : type : runtime : typeg )------------------
.. _typeguard:
   https://github.com/agronholm/typeguard
.. _typeguard.check_type:
   https://typeguard.readthedocs.io/en/latest/userguide.html#checking-types-directly

.. # ------------------( LINKS ~ py : type : runtime : data  )------------------
.. _PyContracts:
   https://github.com/AlexandruBurlacu/pycontracts
.. _contracts:
   https://pypi.org/project/contracts
.. _covenant:
   https://github.com/kisielk/covenant
.. _dpcontracts:
   https://pypi.org/project/dpcontracts
.. _icontract:
   https://github.com/Parquery/icontract
.. _pyadbc:
   https://pypi.org/project/pyadbc
.. _pcd:
   https://pypi.org/project/pcd

.. # ------------------( LINKS ~ py : type : static          )------------------
.. _Pyre:
   https://pyre-check.org
.. _pytype:
   https://github.com/google/pytype

.. # ------------------( LINKS ~ py : type : static : pyright)------------------
.. _pyright:
   https://github.com/Microsoft/pyright
.. _pyright plugins:
   https://github.com/microsoft/pyright/issues/607#issuecomment-873467941
.. _pyright PEP violation #1:
   https://github.com/beartype/beartype/issues/126
.. _pyright PEP violation #2:
   https://github.com/beartype/beartype/issues/127

.. # ------------------( LINKS ~ py : type : static : mypy   )------------------
.. _mypy:
   http://mypy-lang.org
.. _mypy install:
   https://mypy.readthedocs.io/en/stable/getting_started.html
.. _mypy plugin:
   https://mypy.readthedocs.io/en/stable/extending_mypy.html
.. _type narrowing:
   https://mypy.readthedocs.io/en/stable/type_narrowing.html

.. # ------------------( LINKS ~ py : type : tensor          )------------------
.. _nptyping:
   https://github.com/ramonhagenaars/nptyping

.. # ------------------( LINKS ~ soft : ide                  )------------------
.. _PyCharm:
   https://en.wikipedia.org/wiki/PyCharm
.. _Vim:
   https://www.vim.org

.. # ------------------( LINKS ~ soft : ide : vscode         )------------------
.. _Pylance:
   https://github.com/microsoft/pylance-release
.. _VSCode:
   https://code.visualstudio.com
.. _VSCode Mypy extension:
   https://marketplace.visualstudio.com/items?itemName=matangover.mypy

.. # ------------------( LINKS ~ soft : lang                 )------------------
.. _C:
   https://en.wikipedia.org/wiki/C_(programming_language)
.. _C++:
   https://en.wikipedia.org/wiki/C%2B%2B
.. _Ruby:
   https://www.ruby-lang.org
.. _Rust:
   https://www.rust-lang.org

.. # ------------------( LINKS ~ soft : license              )------------------
.. _MIT license:
   https://opensource.org/licenses/MIT

.. # ------------------( LINKS ~ soft : web                  )------------------
.. _React:
   https://reactjs.org
